use chrono::NaiveDateTime;
use rocket::{
    form::Form,
    get, post,
    request::FlashMessage,
    response::{Flash, Redirect},
    routes, FromForm, Route, State,
};
use rocket_dyn_templates::Template;
use sqlx::SqlitePool;
use tera::Context;

use crate::model::{
    boat::Boat,
    logbook::Logbook,
    logtype::LogType,
    user::{AdminUser, User},
};

#[get("/")]
async fn index(
    db: &State<SqlitePool>,
    flash: Option<FlashMessage<'_>>,
    adminuser: AdminUser,
) -> Template {
    let boats = Boat::all(db).await;
    let users = User::cox(db).await;
    let logtypes = LogType::all(db).await;

    let on_water = Logbook::on_water(db).await;
    let completed = Logbook::completed(db).await;

    let mut context = Context::new();
    if let Some(msg) = flash {
        context.insert("flash", &msg.into_inner());
    }

    context.insert("boats", &boats);
    context.insert("users", &users);
    context.insert("logtypes", &logtypes);
    context.insert("loggedin_user", &adminuser.user);
    context.insert("on_water", &on_water);
    context.insert("completed", &completed);

    Template::render("log", context.into_json())
}

#[derive(FromForm)]
struct LogAddForm {
    boat_id: i64,
    shipmaster: i64,
    shipmaster_only_steering: bool,
    departure: String,
    arrival: Option<String>,
    destination: Option<String>,
    distance_in_km: Option<i64>,
    comments: Option<String>,
    logtype: Option<i64>,
}

#[post("/", data = "<data>")]
async fn create(
    db: &State<SqlitePool>,
    data: Form<LogAddForm>,
    _adminuser: AdminUser,
) -> Flash<Redirect> {
    match Logbook::create(
        db,
        data.boat_id,
        data.shipmaster,
        data.shipmaster_only_steering,
        NaiveDateTime::parse_from_str(&data.departure, "%Y-%m-%dT%H:%M").unwrap(), //TODO: fix
        data.arrival
            .clone()
            .map(|a| NaiveDateTime::parse_from_str(&a, "%Y-%m-%dT%H:%M").unwrap()), //TODO: fix
        data.destination.clone(),                                                  //TODO: fix
        data.distance_in_km,
        data.comments.clone(), //TODO: fix
        data.logtype,
    )
    .await
    {
        Ok(_) => Flash::success(Redirect::to("/log"), "Ausfahrt erfolgreich hinzugefügt"),
        Err(_) => Flash::error(Redirect::to("/log"), format!("Fehler beim hinzufügen!"))
    } 
        
    
}

#[derive(FromForm)]
struct LogHomeForm {
    destination: String,
    distance_in_km: i64,
    comments: Option<String>,
    logtype: Option<i64>,
}

#[post("/<logbook_id>", data = "<data>")]
async fn home(
    db: &State<SqlitePool>,
    data: Form<LogHomeForm>,
    logbook_id: i32,
    _adminuser: AdminUser,
) -> Flash<Redirect> {
    let logbook = Logbook::find_by_id(db, logbook_id).await;
    let Some(logbook) = logbook else {
            return Flash::error(
                Redirect::to("/admin/log"),
                format!("Log with ID {} does not exist!", logbook_id),
            )
    };

    match logbook
        .home(
            db,
            &_adminuser.user,
            data.destination.clone(), //TODO: fixme
            data.distance_in_km,
            data.comments.clone(), //TODO: fixme
            data.logtype
        )
        .await
    {
        Ok(_) =>    Flash::success(Redirect::to("/log"), "Successfully updated log"),
        Err(_) =>
         Flash::error(
            Redirect::to("/log"),
            format!("Logbook with ID {} could not be updated!", logbook_id),
        )
    }

}



pub fn routes() -> Vec<Route> {
    routes![index, create, home]
}

#[cfg(test)]
mod test {}
